from unittest import TestCase
from pyage.core import inject
from pyage_forams.solutions.foram import Foram

class TestForam(TestCase):
    def test_step(self):
        inject.config = "pyage_forams.conf.dummy_conf"
        foram = Foram(10)
        # foram.step()


class StaticInsolation(object):
    def __init__(self, surface_insolation=10, insolation_factor=0.2):
        super(StaticInsolation, self).__init__()
        self.surface_insolation = surface_insolation
        self.insolation_factor = insolation_factor

    def get_insolation(self, cell, step=0):
        depth = getattr(cell, "depth", 0)
        return max(0, self.surface_insolation - depth * self.insolation_factor)


class DynamicInsolation(object):
    def __init__(self, seasons):
        super(DynamicInsolation, self).__init__()
        self.seasons = seasons

    def get_insolation(self, cell, step=0):
        depth = getattr(cell, "depth", 0)
        step = step % sum(season[0] for season in self.seasons)
        if step == 0:
            season = self.seasons[-1]
        else:
            s = 0
            while step > self.seasons[s][0]:
                step -= self.seasons[s][0]
                s += 1
            season = self.seasons[s]
        return max(0, season[1] - depth * season[2])



# for testing purposes
from pyage.core import address

from pyage_forams.solutions.genom import GenomFactory
from pyage_forams.solutions.insolation_meter import StaticInsolation


factory = GenomFactory(chambers_limit=5)
genom_factory = lambda: factory.generate

insolation_meter = StaticInsolation
address_provider = address.SequenceAddressProvider
size = lambda: 3


reproduction_minimum = lambda: 10
movement_energy = lambda: 0.25
growth_minimum = lambda: 10
energy_need = lambda: 0.2



from unittest import TestCase
from pyage.core import inject
from pyage_forams.solutions.environment import Cell
from pyage_forams.solutions.insolation_meter import StaticInsolation, DynamicInsolation


class TestInsolation(TestCase):
    def test_insolation(self):
        inject.config = "pyage_forams.conf.dummy_conf"
        insolation_meter = StaticInsolation(surface_insolation=5, insolation_factor=0.1)
        self.assertEqual(insolation_meter.get_insolation(create_cell(), 10), 5)
        self.assertEqual(insolation_meter.get_insolation(create_cell(10), 10), 4)
        self.assertEqual(insolation_meter.get_insolation(create_cell(100), 10), 0)


class TestDynamicInsolation(TestCase):
    def test_insolation(self):
        inject.config = "pyage_forams.conf.dummy_conf"
        insolation_meter = DynamicInsolation([(10, 10, 0.1), (20, 5, 0.5), (1, 1, 1)])
        self.assertEqual(insolation_meter.get_insolation(create_cell(0), 1), 10)
        self.assertEqual(insolation_meter.get_insolation(create_cell(0), 20), 5)
        self.assertEqual(insolation_meter.get_insolation(create_cell(0), 31), 1)


def create_cell(depth=0):
    cell = Cell()
    cell.depth = depth
    return cell




class Thermometer(object):
    def __init__(self, initial_temperature=0):
        super(Thermometer, self).__init__()
        self.temperature = initial_temperature

    def get_temperature(self):
        return self.temperature


class Genom(object):
    def __init__(self, chambers_limit):
        super(Genom, self).__init__()
        self.chambers_limit = chambers_limit


class GenomFactory(object):
    def __init__(self, chambers_limit):
        super(GenomFactory, self).__init__()
        self.life_length = chambers_limit

    def generate(self):
        return Genom(self.life_length)